import os, sys, struct
import fft.map

INDEX1_22 = 0x22
INDEX1_30 = 0x30
INDEX1_70 = 0x70
ARRANGE_0 = 0x0
ARRANGE_1 = 0x1
ARRANGE_2 = 0x2
ARRANGE_3 = 0x3
ARRANGE_4 = 0x4
ARRANGE_5 = 0x5
TIME_0 = 0x0
TIME_1 = 0x1
WEATHER_0 = 0x0
WEATHER_1 = 0x1
WEATHER_2 = 0x2
WEATHER_3 = 0x3
WEATHER_4 = 0x4
DEFAULT_INDEX = (INDEX1_22, ARRANGE_0, TIME_0, WEATHER_0)

RESOURCE_TEXTURE = 0x1701
RESOURCE_TYPE0 = 0x2e01 # Always used with (0x22, 0, 0, 0). Always a big file.
RESOURCE_TYPE1 = 0x2f01 # Always used with (0x30, 0, 0, 0). Usually a big file.
RESOURCE_TYPE2 = 0x3001 # Used with many index combos. Usually a small file.
RESOURCE_EOF = 0x3101


class GNS(object):
    def __init__(self):
        self.file_path = None
        self.file = None
        self.situations = []
        self.items = {}

    def read(self, file_path):
        self.file_path = file_path
        try:
            self.file = open(self.file_path, 'rb')
        except IOError:
            print 'Unable to open file', self.file_path
            sys.exit(1)
        map_dir = os.path.dirname(self.file_path)
        (index1, arrange, temp1, resource_type) = \
                struct.unpack('<HBBH', self.file.read(6))
        situations = {}
        while not resource_type == RESOURCE_EOF:
            time = (temp1 >> 7) & 0x1
            weather = (temp1 >> 4) & 0x7
            self.file.seek(2, 1)
            resource_lba = struct.unpack('<I', self.file.read(4))[0]
            resource_filename = fft.map.lbas[resource_lba]
            resource_file_path = os.path.join(map_dir, resource_filename)
            resource_size = struct.unpack('<I', self.file.read(4))[0]
            situations[(index1, arrange, time, weather)] = True
            if resource_type == RESOURCE_TEXTURE:
                self.items[(index1, arrange, time, weather, 'tex')] = resource_file_path
            else:
                self.items[(index1, arrange, time, weather, 'res')] = resource_file_path
            self.file.seek(4, 1)
            (index1, arrange, temp1, resource_type) = \
                    struct.unpack('<HBBH', self.file.read(6))
        self.situations = sorted(situations.keys())

    def get_texture_files(self, situation):
        (index1, arrange, time, weather) = self.situations[situation]
        search_items = [
                (index1, arrange, time, weather, 'tex'),
                (index1, arrange, TIME_0, WEATHER_0, 'tex'),
                (index1, ARRANGE_0, TIME_0, WEATHER_0, 'tex'),
                (INDEX1_70, ARRANGE_0, TIME_0, WEATHER_0, 'tex'),
                (INDEX1_30, ARRANGE_0, TIME_0, WEATHER_0, 'tex'),
                (INDEX1_22, ARRANGE_0, TIME_0, WEATHER_0, 'tex'),
                ]
        found = []
        for key in search_items:
            if self.items.has_key(key):
                found.append(self.items[key])
        return found

    def get_resource_files(self, situation):
        (index1, arrange, time, weather) = self.situations[situation]
        search_items = [
                (index1, arrange, time, weather, 'res'),
                (index1, arrange, TIME_0, WEATHER_0, 'res'),
                (index1, ARRANGE_0, TIME_0, WEATHER_0, 'res'),
                (INDEX1_70, ARRANGE_0, TIME_0, WEATHER_0, 'res'),
                (INDEX1_30, ARRANGE_0, TIME_0, WEATHER_0, 'res'),
                (INDEX1_22, ARRANGE_0, TIME_0, WEATHER_0, 'res'),
                ]
        found = []
        for key in search_items:
            if self.items.has_key(key):
                found.append(self.items[key])
        return found
